@@ -1,6 +1,6 @@
 import logging
 from pathlib import Path
-from .measures import Measures
+from .measures import ClassificationMeasures
 
 
 log_file = Path(__file__).parents[2] / "graphene.log"
