@@ -47,6 +47,7 @@ _overlap_params = {
 
 class Classifier(BaseLearner):
     def __init__(self, data: pd.DataFrame, cv=10, labels_col=None):
+        super().__init__()
         self.logger = logging.getLogger(__name__)
 
         if labels_col is None:
@@ -78,11 +79,6 @@ class Classifier(BaseLearner):
         y_true = y_true[:, classes_order.argsort()]
         return self._call_function(module=metrics, name=metric, y_true=y_true, y_pred=y_pred), y_pred[y_true == 1]
 
-    def estimate_ih(self):
-        L = len(self.predicted_proba.columns)
-        IH = 1 - (1/L)*self.predicted_proba.sum(axis=1).values
-        return IH
-
     @staticmethod
     def update_params(new_params=None):
         if new_params is None:
@@ -152,7 +148,7 @@ class Classifier(BaseLearner):
 
                 if hyper_param_optm:
                     self.logger.info("Optimizing classifier hyper-parameters")
-                    best_params = find_best_params(name=hpo_name, clf=algo, fixed_params=kwargs,
+                    best_params = find_best_params(name=hpo_name, predictor=algo, fixed_params=kwargs,
                                                    X=X_train, y=y_train,
                                                    max_evals=hpo_evals, hpo_timeout=hpo_timeout)
                     self.logger.debug(f"Best hyper-parameters found: {best_params}")
