@@ -1,3 +1,4 @@
+import sys
 import argparse
 import logging
 import shutil
@@ -14,7 +15,7 @@ from . import integrator, formatter
 from .context import Configuration, Workspace
 from .feature_selection import featfilt
 from .metrics import loss_threshold
-from .visualization import Demo, App
+from .visualization import Demo, AppClassification
 
 
 _conf_file = 'config.yaml'
@@ -42,20 +43,25 @@ def main(args):
                 if abs_path.exists():
                     conf.set(name, str(abs_path))
                 else:
-                    logger.error("Invalid '{0}': '{1}'".format(name, abs_path))
-                    raise NotADirectoryError
+                    logger.error("Invalid '{0}': '{1}'.".format(name, abs_path))
+                    sys.exit(1)
 
         file_path = Path(conf.get('datafile'))
         if file_path.is_file():
             logger.info("Reading input dataset: '{0}'".format(file_path))
             df_dataset = pd.read_csv(file_path)
         else:
-            logger.error("Invalid datafile '{0}'".format(file_path))
-            raise FileNotFoundError
+            logger.error("Invalid datafile '{0}'.".format(file_path))
+            sys.exit(1)
 
         kwargs = conf.get_full()
         rootdir_path = Path(conf.get('rootdir'))
-        logger.info(f"Type of problem: '{conf.get('problem')}'")
+        problem = str.lower(conf.get('problem'))
+        if problem in {'classification', 'regression'}:
+            logger.info(f"Type of problem: '{problem}'")
+        else:
+            logger.error(f"Unknown problem type '{problem}'.")
+            sys.exit(1)
 
         if args.meta:
             logger.info("Building metadata.")
@@ -112,7 +118,8 @@ def main(args):
             elif isa_engine == 'matlab_compiled':
                 integrator.run_matilda_module(rootdir=rootdir_path)
             else:
-                raise RuntimeError(f"Unknown ISA engine {repr(isa_engine)}.")
+                logger.error(f"Unknown ISA engine '{repr(isa_engine)}'.")
+                sys.exit(1)
 
         if args.app:
             logging.getLogger().setLevel(logging.WARNING)
@@ -120,8 +127,12 @@ def main(args):
             ws = Workspace(rootdir_path, file_path)
             ws.load()
 
-            app = App(ws)
-            app.show(port=5001, websocket_origin=['127.0.0.1:5001', 'localhost:5001'], open=args.browser)
+            if problem == 'classification':
+                app = AppClassification(ws)
+            elif problem == 'regression':
+                # app = AppRegression(ws)
+                raise NotImplementedError("Regression problems not yet supported. Coming soon!")
+            app.show(port=5001, show=args.browser)
 
         end = time.time()
         elapsed_time = end - start
@@ -130,28 +141,29 @@ def main(args):
         else:
             logger.info(f"Total elapsed time: {int(elapsed_time//60)}m{int((elapsed_time/60 - elapsed_time//60)*60)}s")
         logger.info("Instance Hardness analysis finished.")
+        sys.exit(0)
 
 
 def cli():
     parser = argparse.ArgumentParser(description="PyHard - Python Instance Hardness Framework. \n"
                                                  "If you find a bug, please open an issue in our repo: "
                                                  "https://gitlab.com/ita-ml/pyhard/-/issues")
+    parser.add_argument('-F', '--files', dest='generate', action='store_true', default=False,
+                        help="generate configuration files locally")
+    parser.add_argument('--app', dest='app', action='store_true', default=False,
+                        help="run app to visualize data")
+    parser.add_argument('--demo', dest='demo', action='store_true', default=False,
+                        help="run demo for toy datasets")
     parser.add_argument('--no-meta', dest='meta', action='store_false',
                         help="does not generate a new metadata file; uses previously saved instead")
     parser.add_argument('--no-isa', dest='isa', action='store_false',
                         help="does not execute the instance space analysis")
     parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', default=False,
                         help="verbose mode")
-    parser.add_argument('--app', dest='app', action='store_true', default=False,
-                        help="run app to visualize data")
     parser.add_argument('--no-browser', dest='browser', action='store_false', default=True,
                         help="run app without opening browser")
-    parser.add_argument('--demo', dest='demo', action='store_true', default=False,
-                        help="run demo for datasets in 'data/' directory")
     parser.add_argument('-c', '--config', dest='other', default=None, required=False,
                         metavar='FILE', help="specifies a path to a config file other than default")
-    parser.add_argument('-F', '--files', dest='generate', action='store_true', default=False,
-                        help="generate configuration files locally")
 
     args = parser.parse_args()
     print("run 'pyhard --help' to see all options.")
