@@ -1,10 +1,10 @@
 import pandas as pd
 import numpy as np
-import rankaggregation as ra
 from scipy.special import softmax
 from sklearn.feature_selection import VarianceThreshold
 from .thirdparty import skfeature
 from .utils import call_module_func
+from .thirdparty import rank_aggregation as ra
 
 
 def _select(F, J, how='cumsum', **kwargs):
@@ -40,8 +40,7 @@ def _prefilter(X, var_threshold=1e-3):
     return sel.get_support()
 
 
-def featfilt(df_metadata: pd.DataFrame, max_n_features=10, method='icap',
-             eta=0.8, var_filter=True, var_threshold=0, **kwargs):
+def featfilt(df_metadata: pd.DataFrame, max_n_features=10, method='icap', var_filter=True, var_threshold=0, **kwargs):
     """
     Supervised feature filtering function. It involves three steps:
 
@@ -62,8 +61,6 @@ def featfilt(df_metadata: pd.DataFrame, max_n_features=10, method='icap',
     :type max_n_features: int
     :param method: score method (see :py:mod:`pyhard.thirdparty.skfeature` module)
     :type method: str
-    :param eta: cumulative score threshold (between 0 and 1)
-    :type eta: float
     :param var_filter: enables variance filter
     :type var_filter: bool
     :param var_threshold: variance filter threshold
